//package org.enhydra.jdbc.standard;
import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.sql.*;
import javax.transaction.xa.*;
//import org.enhydra.jdbc.standard.*;

public class J2xaTest extends J2poolTest {

	static {
		dataName = "jdbc/xapool";
		dataSourceClass = "org.enhydra.jdbc.standard.StandardXADataSource";
	} // class constructor

	TestXid xid;
	int xidCount;
	
	public J2xaTest () {
		super ();
	} // method J2xaTest


	public Connection getConnection () throws SQLException {
		return getConnection (null, null);
	} // method getConnection

	public Connection getConnection (String user, String passwd) throws SQLException {
		Connection ret = null;
		PooledConnection con=getPooledConnection(user, passwd, true);

		try {
			// Return an XADelegateConnection to the user
			ret = con.getConnection();

			// We know that the above is really an XAConnection
			// so we can get it's XAResource
			XAConnection xacon = (XAConnection)con;
			XAResource xar = xacon.getXAResource();
			xar.setTransactionTimeout (20);

			// Associate the connection with a global transaction
			int flags=0;
			if (xid==null) {
				xid = new TestXid();
				flags=XAResource.TMNOFLAGS;
			} else {
				flags=XAResource.TMJOIN;
			} // if-else
			xidCount++;
			xar.start (xid, flags);

		} catch (Exception e) {
			e.printStackTrace();
		} // try-catch

		return ret;
	} // method getConnection

	public void connectionClosed(ConnectionEvent event) {
		try {
			Object obj = event.getSource();
			pool.addElement (obj);

			XAConnection xacon = (XAConnection)obj;
			if (xidCount==1) {
				// last use of this xid
				XAResource xar = xacon.getXAResource();
				xar.end (xid, XAResource.TMSUCCESS);
				int res = xar.prepare (xid);
				if (res != XAResource.XA_RDONLY) {
					xar.commit (xid, false);
				} // if
			} // if
		} catch (SQLException sql) {
			sql.printStackTrace();
		} catch (XAException e) {
			System.out.println (e.errorCode);
			e.printStackTrace();
		} finally {
			if (--xidCount==0) {
				xid=null;
			} // if
		} // try-catch
	}

	/**
	 * The factory interface.
	 */
	public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {
		Reference ref = (Reference)refObj;
		if (ref.getClassName().equals(getClass().getName())) {
			J2xaTest dataSource = new J2xaTest ();
			dataSource.setDriverName((String)ref.get("driverName").getContent());
			dataSource.setUrl((String)ref.get("url").getContent());
			return dataSource;
		} else {
			return null;
		}
	}

} // class J2xaTest

// A trivial little Xid implementation.
class TestXid implements Xid {
	
	byte[] branch;
	byte[] global;

	static byte id;

	TestXid () {
		branch = new byte[1];
		branch[0] = id++;
		global = new byte[1];
		global[0] = id++;
	} // method TestXid

	public boolean equals (Object obj) {
		TestXid xid = (TestXid)obj;
		if ((branch[0] != xid.branch[0]) ||
			(global[0] != xid.global[0])) {
			return false;
		} // if
		return true;
	} // method equals

	public String toString () {
		StringBuffer buff = new StringBuffer (64);
		buff.append ("Global=");
		buff.append (global[0]);
		buff.append (" branch=");
		buff.append (branch[0]);
		return buff.toString();
	} // method toString

	public byte[] getBranchQualifier() {return branch;}
	public byte[] getGlobalTransactionId () {return global;}
	public int getFormatId() {return 0;}
  
} // class TestXid 